/*
** Module   : ClientParameters.java
** Abstract : Container of various client-side parameters which need to be exposed to the server.
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------------------Description---------------------------------
** 001 CA  20131004 Created initial version. 
*/

package com.goldencode.p2j.main;

import java.io.*;

/**
 * Container of client-side parameters which need to be exposed to the server-side application 
 * code.
 */
public class ClientParameters
implements Externalizable
{
   /**
    * Constant identifying the "TTY" window-system. Use this constant as a default for all CHUI 
    * drivers.
    */
   public static final String WINDOW_SYSTEM_TTY = "TTY";
   
   /** Constant identifying the "MS-WINDOWS" window-system. */
   public static final String WINDOW_SYSTEM_MS_WINDOWS = "MS-WINDOWS";

   /** Constant identifying the "MS-WIN95" window-system. */
   public static final String WINDOW_SYSTEM_MS_WIN95 = "MS-WIN95";

   /**
    * Constant identifying the "MS-WINXP" window-system. Use this constant as a default for all
    * GUI drivers.
    */
   public static final String WINDOW_SYSTEM_MS_WINXP = "MS-WINXP";

   /** Identify the SESSION:WINDOW-SYSTEM attribute used by this client. */
   private String windowSystem = null;
   
   /**
    * Getter for the {@link #windowSystem} parameter.
    * 
    * @return   See above.
    */
   public String getWindowSystem()
   {
      return windowSystem;
   }

   /**
    * Setter for the {@link #windowSystem} parameter.
    * 
    * @param    windowSystem
    *           The value for the {@link #windowSystem} parameter.
    */
   public void setWindowSystem(String windowSystem)
   {
      this.windowSystem = windowSystem;
   }

   /**
    * Replacement for the default object writing method. The latest state is written to the output
    * destination.
    * 
    * @param    out
    *           The output destination to which parameters will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeObject(windowSystem);
   }

   /**
    * Replacement for the default object reading method. The latest state is read from the input
    * source.
    * 
    * @param    in
    *           The input source from which parameters will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If a parameter can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      windowSystem = (String) in.readObject();
   }
}
