/*
** Module   : ConsoleDriver.java
** Abstract : Text Console-based driver.
**
** Copyright (c) 2010-2011, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130723 Created initial version
** 002 CA  20131004 Implement the WINDOW-SYSTEM attribute for this driver.
*/

package com.goldencode.p2j.ui.client.chui.driver.console;

import java.util.*;

import com.goldencode.p2j.main.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.driver.*;

/**
 * Text console-based driver. The underlying native implementation is used as backend to provide
 * required features. The example is NCURSES package for Linux or console API wor Windows OS.
 */
public class ConsoleDriver
implements ScreenDriver
{
   /** Special key code which denotes mouse event. */
   private static final int KEY_MOUSE = 0631;

   /** Reference to console low-level driver. */
   private ConsoleHelper helper = ConsoleHelper.getDefaultConsoleHelper();

   /** Direct to screen output primitives. */
   private OutputPrimitives direct;

   /** Key processor. */
   private KeyProcessor processor = new KeyProcessor();

   /** CHUI-specific widget factory. */
   private WidgetFactory factory = new ChuiWidgetFactory();

   /**
    * Constructor. 
    */
   public ConsoleDriver()
   {
   }

   /**
    * Generate a beep signal. 
    * <p>
    * Implementation deoends on native layer and (in turn) from particular 
    * terminal. If terminal does not support this feature or sound in terminal
    * is turned off, there will be no audible feedback to user.
    */
   @Override
   public void beep()
   {
      helper.beep();
   }

   /**
    * Clear screen.
    */
   @Override
   public void clear()
   {
      helper.clear();
   }

   /**
    * Get instance of {@link OutputPrimitives} which provides direct access to 
    * required functionality of attached terminal.
    */
   @Override
   public OutputPrimitives getPrimitives()
   {
      return direct;
   }

   /**
    * Check if terminal supports colors. 
    * <p>
    * This implementation also checks if colors are enabled in low-level 
    * driver. If they are not (and usually this is so), then application 
    * will run in B/W mode even on color terminal.
    */
   @Override
   public boolean hasColors()
   {
      return helper.hasColors() && ConsoleHelper.isColorEnabled;
   }

   /**
    * Read the key from terminal and translate it into P2J internal 
    * representation. Translation details are in {@link KeyProcessor} class. 
    * Mouse events (which also returned from native layer) are filtered out 
    * because mouse is not supported.  
    */
   @Override
   public int readKey()
   {
      int key = helper.readKey();

      if (key == ConsoleDriver.KEY_MOUSE || key == -1)
         return -1;

      return processor.map(key);
   }

   /**
    * Reset screen mode.
    */
   @Override
   public void resetMode()
   {
      helper.resetMode();
   }

   /**
    * Resume terminal after {@link #suspend()} call.
    */
   @Override
   public void resume()
   {
      helper.resume();
   }

   /**
    * Set cursor status (ON/OFF).
    */
   @Override
   public boolean setCursorStatus(boolean on) throws IllegalStateException
   {
      return helper.setCursorStatus(on);
   }

   /**
    * Set terminal type according to passed string. 
    */
   @Override
   public void setTerminalType(String name)
   {
      ConsoleHelper.setTermType(name);
   }

   /**
    * Suspend terminal and release it for use by other applications.
    */
   @Override
   public void suspend()
   {
      helper.suspend();
   }

   /**
    * Get terminal type string.
    */
   @Override
   public String terminalType()
   {
      return ConsoleHelper.getTermType();
   }

   /**
    * Initialize driver.
    */
   @Override
   public void init()
   {
      ConsoleHelper.initChui();
      direct = new ConsolePrimitives(helper);

      String type = terminalType();

      if ("vt220".equalsIgnoreCase(type))
         processor.setDelWorkaroundActive(true);
   }
   
   /**
    * Add colors to mapping.
    *
    * @param   colors
    *          Array of colors added to mapping since last call.
    */
   @Override
   public void addColors(List<Color> colors)
   {
      // at present this functionality is not supported
   }

   /**
    * Get driver-specific widget factory.
    * 
    * @return  reference to widget factory.
    */
   @Override
   public WidgetFactory getFactory()
   {
      return factory;
   }
   
   /**
    * Get the default WINDOW-SYSTEM used by this driver, which is 
    * {@link ClientParameters#WINDOW_SYSTEM_TTY}.
    * 
    * @return   Always the .{@value ClientParameters#WINDOW_SYSTEM_TTY} string.
    */
   @Override
   public String getWindowSystem()
   {
      return ClientParameters.WINDOW_SYSTEM_TTY;
   }
}
