/*
** Module   : SwingChuiDriver.java
** Abstract : Swing-based ChUI driver.
**
** Copyright (c) 2011-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20110914 Created initial version.
** 002 SIY 20111202 Updated imports.
** 003 SIY 20111221 Added widget factory.
** 004 CA  20131004 Implement the WINDOW-SYSTEM attribute for this driver.
*/

package com.goldencode.p2j.ui.client.chui.driver.swing;

import java.util.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.main.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.driver.*;

/**
 * Swing-based ChUI driver.
 */
public class SwingChuiDriver
implements ScreenDriver
{
   /** Direct to screen output primitives. */
   private OutputPrimitives direct;

   /** Terminal simulator implemented in Swing. */
   private ChuiSimulator sim = null;
   
   /** Bogus terminal type name. */
   private String termType = "vt100";

   /** This driver-specific widget factory */
   private WidgetFactory factory = new ChuiWidgetFactory();
   
   /**
    * Constructor.
    *
    * @param    sim
    *           Terminal simulator instance used for direct drawing.
    */
   public SwingChuiDriver(ChuiSimulator sim)
   {
      this.sim = sim;
   }
   
   /**
    * Read configuration values from the bootstrap configuration and convert
    * these into an options instance.
    * <p>
    * The following parameters are supported:
    * <p>
    * <pre>
    * Category:Group:Key       Type
    * -----------------------  ---------------------------
    * client:chui:rows         integer
    * client:chui:columns      integer
    * client:chui:background   text which can be converted to a Color by Color.decode()
    * client:chui:foreground   text which can be converted to a Color by Color.decode()
    * client:chui:selection    text which can be converted to a Color by Color.decode()
    * client:chui:fontname     text containing a monospaced (fixed width) font family name
    * client:chui:fontsize     integer
    * </pre>
    *
    * @param    cfg
    *           The bootstrap configuration.
    *
    * @return   A new options instance with any defaults overridden as specified
    *           in the bootstrap configuration.
    */
   public static TerminalOptions convertToOptions(BootstrapConfig cfg)
   {
      TerminalOptions options = new TerminalOptions();
      
      String next;
      
      if ((next = cfg.getString("client", "chui", "rows", null)) != null)
      {
         options.rows = parseInt(next, options.rows);
      }
      
      if ((next = cfg.getString("client", "chui", "columns", null)) != null)
      {
         options.columns = parseInt(next, options.columns);
      }
      
      if ((next = cfg.getString("client", "chui", "background", null)) != null)
      {
         options.background = parseColor(next, options.background);
      }
      
      if ((next = cfg.getString("client", "chui", "foreground", null)) != null)
      {
         options.foreground = parseColor(next, options.foreground);
      }
      
      if ((next = cfg.getString("client", "chui", "selection", null)) != null)
      {
         options.selection = parseColor(next, options.selection);
      }
      
      if ((next = cfg.getString("client", "chui", "fontname", null)) != null)
      {
         options.fontname = next;
      }
      
      if ((next = cfg.getString("client", "chui", "fontsize", null)) != null)
      {
         options.fontsize = parseInt(next, options.fontsize);
      }
      
      return options;
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#beep()
    */
   @Override
   public void beep()
   {
      java.awt.Toolkit.getDefaultToolkit().beep();
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#clear()
    */
   @Override
   public void clear()
   {
      sim.clear();
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#getPrimitives()
    */
   @Override
   public OutputPrimitives getPrimitives()
   {
      return direct;
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#hasColors()
    */
   @Override
   public boolean hasColors()
   {
      return true;
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#readKey()
    */
   @Override
   public int readKey()
   {
      return sim.readKey();
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#resetMode()
    */
   @Override
   public void resetMode()
   {
      // NOP for now, what should this do?
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#resume()
    */
   @Override
   public void resume()
   {
      // NOP for now, what should this do?
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#setCursorStatus(boolean)
    */
   @Override
   public boolean setCursorStatus(boolean on)
   throws IllegalStateException
   {
      return sim.setCursorStatus(on);
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#setTerminalType(java.lang.String)
    */
   @Override
   public void setTerminalType(String name)
   {
      termType = name;
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#suspend()
    */
   @Override
   public void suspend()
   {
      // NOP for now, what should this do?
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#terminalType()
    */
   @Override
   public String terminalType()
   {
      return termType;
   }

   /**
    * @see com.goldencode.p2j.ui.client.driver.ScreenDriver#init()
    */
   @Override
   public void init()
   {
      direct = new SwingChuiPrimitives(sim);
   }
   
   /**
    * Add colors to mapping.
    *
    * @param    colors
    *           Array of colors added to mapping since last call.
    */
   @Override
   public void addColors(List<Color> colors)
   {
      // at present this functionality is not supported
   }
   
   /**
    * Attempt to convert the given string into an integer using
    * <code>Integer.parseInt</code>.
    *
    * @param    spec
    *           String encoded number specification.
    * @param    def
    *           Default value to return if the spec cannot be parsed properly.
    *
    * @return   The parsed number or the default if there is any parsing
    *           problem.
    */
   private static int parseInt(String spec, int def)
   {
      int val = def;
      
      try
      {
         val = Integer.parseInt(spec);
      }
      
      catch (NumberFormatException nfe)
      {
         // ignore
      }
      
      return val;
   }
   
   /**
    * Attempt to convert the given string into a color using
    * <code>Color.decode</code>.
    *
    * @param    spec
    *           String encoded color specification.
    * @param    def
    *           Default value to return if the spec cannot be parsed properly.
    *
    * @return   The parsed color or the default if the specification was
    *           invalid.
    */
   private static java.awt.Color parseColor(String spec, java.awt.Color def)
   {
      java.awt.Color color = def;
      
      try
      {
         color = java.awt.Color.decode(spec);
      }
      
      catch (NumberFormatException nfe)
      {
         // ignore
      }
      
      return color;
   }

   /**
    * Get driver-specific widget factory.
    * 
    * @return  reference to widget factory.
    */
   @Override
   public WidgetFactory getFactory()
   {
      return factory ;
   }
   
   /**
    * Get the default WINDOW-SYSTEM used by this driver, which is 
    * {@link ClientParameters#WINDOW_SYSTEM_TTY}.
    * 
    * @return   Always the .{@value ClientParameters#WINDOW_SYSTEM_TTY} string.
    */
   @Override
   public String getWindowSystem()
   {
      return ClientParameters.WINDOW_SYSTEM_TTY;
   }
}
