/*
** Module   : CommonSession.java
** Abstract : Define all APIs available via the SESSION system handle.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130211 Moved first/last procedure attribute to a separate interface. 
** 003 CA  20130221 Added Sizeable support.
** 004 CA  20130222 Added more SESSION-related attributes and methods.
** 005 OM  20130607 Added SESSION:YEAR-OFFSET attribute support.
** 006 CA  20130927 Added LegacyResource annotation, to determine resource type. Implemented the 
**                  following attributes: PARAMETER, FIRST-QUERY, FIRST-/LAST-SERVER, 
**                  FIRST-/LAST-SOCKET, FIRST-/LAST-SERVER-SOCKET.
** 007 CA  20131004 WINDOW-SYSTEM is converted to getWindowSystem instead of getWindowsVersion.
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes from the SYSTEM handle:
 * <table>
 * <tr>
 *    <th>SESSION attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>BATCH-MODE</td>
 *    <td>{@link #isBatchMode()}</td>
 * </tr>
 * <tr>
 *    <td>CHARSET</td>
 *    <td>{@link #getCharset()}</td>
 * </tr>
 * <tr>
 *    <td>DATE-FORMAT</td>
 *    <td>{@link #getDateOrder()}, {@link #setDateOrder(character)} and 
 *        {@link #setDateOrder(String)}
 *    </td>
 * </tr>
 * <tr>
 *    <td>DISPLAY-TYPE</td>
 *    <td>{@link #getDisplayType()}</td>
 * </tr>
 * <tr>
 *    <td>HEIGHT-CHARS</td>
 *    <td>emitted as getHeightChars and setHeightChars, not implemented</td>
 * </tr>
 * <tr>
 *    <td>HEIGHT-PIXELS</td>
 *    <td>{@link #getHeightPixels()} and setHeightPixels</td>
 * </tr>
 * <tr>
 *    <td>TEMP-DIRECTORY</td>
 *    <td>{@link #getTempDirectory()}</td>
 * </tr>
 * <tr>
 *    <td>WIDTH-CHARS</td>
 *    <td>emitted as getWidthChars and setWidthChars, not implemented</td>
 * </tr>
 * <tr>
 *    <td>WIDTH-PIXELS</td>
 *    <td>{@link #getWidthPixels()} and setWidthPixels</td>
 * </tr>
 * <tr>
 *    <td>LAST-PROCEDURE</td>
 *    <td>{@link #lastProcedure()}</td>
 * </tr>
 * <tr>
 *    <td>FIRST-PROCEDURE</td>
 *    <td>{@link #firstProcedure()}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapSession(handle)} API, which, depending on the 
 * handle's state, it either returns an {@link SessionUtils#asHandle()} 
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy}. 
 */
@LegacyResource(resource = LegacyResource.PSEUDO_WIDGET)
public interface CommonSession
extends CommonProcedure,
        CommonHandle,
        FirstLastProcedureAttribute,
        Sizeable
{
   /**
    * Reports if the current context is running in batch mode.
    * <p>
    * The value returned may have been found via a search algorithm that
    * is account (user or process) specific or group specific
    * within the current server or a global default for all servers. 
    * <p>
    * The implementation iteratively looks up the directory node under:
    * /server/&lt;serverID&gt;/runtime/&lt;account_or_group&gt;/batchMode
    * <p>
    * If no user/process or group nodes are present, then this is checked:
    * /server/&lt;serverID&gt;/runtime/default/batchMode
    * <p>
    * If no /server/&lt;serverID&gt;/runtime node exists, this is checked
    * (it is the global default area for all servers):
    * /server/default/runtime/&lt;account_or_group&gt;/batchMode
    * <p>
    * Finally, if no user/process or group nodes are present in the global
    * default area, then this is checked:
    * /server/default/runtime/default/batchMode
    * <p>
    * If no value is found via this lookup, then the default value of
    * <code>false</code> will be returned.
    *
    * @return   <code>true</code> if the current context is in batch mode.
    */
   public logical isBatchMode();
   
   /**
    * Reports the default character set name. Since the Progress default
    * charset is ISO-8859-1, when the JVM reports that UTF-8 is the default,
    * this method will return the string "ISO-8859-1" instead.
    * <p>
    * The assumption (possibly not always correct) is that UTF-8 is not a
    * valid Progress charset and instead, to provide a proper default, we
    * must override with a compatible charset. Note that if the default
    * charset name is not UTF-8, it is assumed that the charset has been
    * overriden already (at JVM startup).  In that case, the default is
    * returned unchanged.
    *
    * @return   A default charset name.
    */
   public character getCharset();

   /** 
    * Gets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components is a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value accessed is specific to the current user's context.
    *
    * @return   Defines the order of date components for conversion to/from
    *           strings.
    */
   public character getDateOrder();
   
   /** 
    * Sets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components as a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value set is specific to the current user's context.
    *
    * @param    dateFormat
    *           Defines the order of date components for conversion to/from
    *           strings.
    */
   public void setDateOrder(String dateFormat);
   
   /** 
    * Sets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components as a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value set is specific to the current user's context.
    *
    * @param    dateFormat
    *           Defines the order of date components for conversion to/from strings.
    */
   public void setDateOrder(character dateFormat);
   
   /**
    * Implementation of SESSION:YEAR-OFFSET attribute getter. Returns the 
    * currently configured start year for the two-digit Y2K windowing range.
    * This is the equivalent to the Progress -yy command line parameter. The 
    * default value is 1950 unless otherwise specified in the directory (see
    * {@link date#getWindowingYear}).
    * 
    * @return  The start year for the two-digit windowing range.
    */
   public integer getYearOffset();
   
   /**
    * Implementation of SESSION:YEAR-OFFSET attribute setter. Sets the 
    * configured start year for the two-digit Y2K windowing range. This is
    * the equivalent of the Progress -yy command line parameter. The 
    * default value is 1950 unless otherwise specified in the directory (see
    * {@link date#getWindowingYear}).
    * 
    * @param   year
    *          The new start year for the two-digit windowing range.
    */
   public void setYearOffset(NumberType year);
   
   /**
    * Gets the display type (GUI or TTY).  Corresponds with the
    * <code>SESSION:DISPLAY-TYPE</code> system handle attribute.
    * 
    * @return  Display type.  Currently, only TTY is supported.
    */
   public character getDisplayType();

   /**
    * Get the width in pixels of the display.  Corresponds with the
    * <code>SESSION:WIDTH-PIXELS</code> system handle attribute.
    * 
    * @return  Display width in pixels.
    */
   public integer getWidthPixels();

   /**
    * Get the height in pixels of the display.  Corresponds with the
    * <code>SESSION:HEIGHT-PIXELS</code> system handle attribute.
    * 
    * @return  Display height in pixels.
    */
   public integer getHeightPixels();
   
   /**
    * Gets the name of the directory where temporary files are stored.
    * Corresponds with with the <code>SESSION:TEMP-DIRECTORY</code> system
    * handle attribute.
    * 
    * @return  Name of temporary directory.
    */
   public character getTempDirectory();
   
   /**
    * Method that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be
    *           exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public logical export(String attrList);
   
   /**
    * Method that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be
    *           exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public logical export(character attrList);
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of
    * the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return   the corresponding SERVER-OPERATING-MODE attribute of the session 
    *           handle.
    */
   public character getServerOperatingMode();
   
   /**
    * Indicates if the session is an AppServer session or not.
    * 
    * @return   <code>true</code> if the current session is an AppServer session
    *           or false otherwise.
    */
   public logical isRemote();

   /**
    * Get the value of CLIENT-TYPE attribute, one of the following values:
    * <ul>
    *    <li>Standard ABL client 4GL: CLIENT</li>
    *    <li>WebClient: WEBCLIENT</li>
    *    <li>AppServer agent: APPSERVER</li>
    *    <li>WebSpeed agent: WEBSPEED</li>
    *    <li>Other special-purpose clients: Unknown value (?)</li>
    * </ul>
    * 
    * @return   See above.
    */
   public character getClientType();

   /**
    * Returns the handle for the first created dynamic query.
    * 
    * @return   See above.
    */
   public handle firstQuery();

   /**
    * Get the FIRST-SERVER attribute of this session.
    * 
    * @return   See above.
    */
   public handle firstServer();
   
   /**
    * Get the FIRST-SERVER-SOCKET attribute of this session.
    * 
    * @return   See above.
    */
   public handle firstServerSocket();
   
   /**
    * Get the FIRST-SOCKET attribute of this session.
    * 
    * @return   See above.
    */
   public handle firstSocket();

   /**
    * Returns the handle for the first dynamic buffer in the first table containing a dynamic 
    * buffer.
    * 
    * @return   See above.
    */
   public handle firstBuffer();

   /**
    * Get the NUMERIC-SEPARATOR attribute, which is the character that represents a number's 
    * thousands separator, in its formatted representation.
    * 
    * @return   See above.
    */
   public character getNumericSeparator();

   /**
    * Get the NUMERIC-DECIMAL-POINT attribute, which is the character that represents a number's
    * decimal point, in its formatted representation.
    * 
    * @return   See above.
    */
   public character getNumericDecimalPoint();

   /**
    * Get the NUMERIC-FORMAT attribute, which provides the way of interpreting commas and periods
    * within numeric values.
    * <p>
    * The possible values are "American", "European" or a character string consisting of the 
    * thousands separator followed by the decimal point. This attribute provides the same 
    * functionality as the European Numeric Format (-E) parameter.
    * 
    * @return   See above.
    */
   public character getNumericFormat();

   /**
    * Set the NUMERIC-FORMAT attribute. See {@link #getNumericFormat()} for its representation.
    * 
    * @param    format
    *           The new value.
    */
   public void setNumericFormat(String format);

   /**
    * Set the NUMERIC-FORMAT attribute. See {@link #getNumericFormat()} for its representation.
    * 
    * @param    format
    *           The new value.
    */
   public void setNumericFormat(Text format);

   /**
    * Get the value of the PARAMETER attribute, which is set to the value of the Parameter 
    * (-param) startup parameter specified for the current session.
    * 
    * @return   See above.
    */
   public character getParameter();

   /**
    * Get the value of the WINDOW-SYSTEM attribute, which is the version on which the client was 
    * started.
    * 
    * @return   See above.
    */
   public character getWindowSystem();

   /**
    * Returns a string indicating the current wait-state; valid values are:
    * <ul>
    *    <li>GENERAL</li>
    *    <li>COMPILER
    *    <li>CUSTOM - if SET-WAIT-STATE was not called with <code>GENERAL</code> or 
    *        <code>COMPILER</code> constants.</li>
    *    <li>empty string, if none of the above.</li>
    * </ul>
    * 
    * @return   See above.
    */
   public character getWaitState();

   /**
    * Sets the NUMERIC-SEPARATOR and NUMERIC-DECIMAL-POINT attributes simultaneously.
    * 
    * @param    separator
    *           The new NUMERIC-SEPARATOR value.
    * @param    decPoint
    *           The new NUMERIC-DECIMAL-POINT value.
    *           
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setNumericFormat(String separator, String decPoint);

   /**
    * Sets the NUMERIC-SEPARATOR and NUMERIC-DECIMAL-POINT attributes simultaneously.
    * 
    * @param    separator
    *           The new NUMERIC-SEPARATOR value.
    * @param    decPoint
    *           The new NUMERIC-DECIMAL-POINT value.
    *           
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setNumericFormat(character separator, String decPoint);

   /**
    * Sets the NUMERIC-SEPARATOR and NUMERIC-DECIMAL-POINT attributes simultaneously.
    * 
    * @param    separator
    *           The new NUMERIC-SEPARATOR value.
    * @param    decPoint
    *           The new NUMERIC-DECIMAL-POINT value.
    *           
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setNumericFormat(String separator, character decPoint);

   /**
    * Sets the NUMERIC-SEPARATOR and NUMERIC-DECIMAL-POINT attributes simultaneously.
    * 
    * @param    separator
    *           The new NUMERIC-SEPARATOR value.
    * @param    decPoint
    *           The new NUMERIC-DECIMAL-POINT value.
    *           
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setNumericFormat(character separator, character decPoint);

   /**
    * The value of state-string determines the wait state and the type of wait cursor displayed. 
    * The valid values are: 
    * <ul>
    *    <li>GENERAL - Displays the system-busy cursor</li>
    *    <li>COMPILER - Displays the ABL compiler cursor</li>
    * </ul>
    * 
    * @param    state
    *           The new wait-state value.
    * 
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setWaitState(String state);

   /**
    * The value of state-string determines the wait state and the type of wait cursor displayed. 
    * The valid values are: 
    * <ul>
    *    <li>GENERAL - Displays the system-busy cursor</li>
    *    <li>COMPILER - Displays the ABL compiler cursor</li>
    * </ul>
    * 
    * @param    state
    *           The new wait-state value.
    * 
    * @return   <code>true</code> if the operation was successful.
    */
   public logical setWaitState(character state);
   
   /**
    * Get the LAST-SERVER attribute of this session.
    * 
    * @return   See above.
    */
   public handle lastServer();

   /**
    * Get the LAST-SERVER-SOCKET attribute of this session.
    * 
    * @return   See above.
    */
   public handle lastServerSocket();

   /**
    * Get the LAST-SOCKET attribute of this session.
    * 
    * @return   See above.
    */
   public handle lastSocket();
}